import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_mentions/flutter_mentions.dart';

void main() {
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      builder: (_, child) => Portal(child: child),
      title: 'Flutter Demo',
      theme: ThemeData(
        primarySwatch: Colors.blue,
        visualDensity: VisualDensity.adaptivePlatformDensity,
      ),
      home: MyHomePage(title: 'Flutter Demo Home Page'),
    );
  }
}

class MyHomePage extends StatefulWidget {
  MyHomePage({Key key, this.title}) : super(key: key);

  final String title;

  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  GlobalKey<FlutterMentionsState> key = GlobalKey<FlutterMentionsState>();
  bool _showList = false;
  final _users = [
    {
      'id': '61as61fsa',
      'display': 'fayeedP',
      'full_name': 'Fayeed Pawaskar',
      'photo':
          'https://images.pexels.com/photos/220453/pexels-photo-220453.jpeg?auto=compress&cs=tinysrgb&dpr=2&h=650&w=940'
    },
    {
      'id': '61asasgasgsag6a',
      'display': 'khaled',
      'full_name': 'DJ Khaled',
      'style': TextStyle(color: Colors.purple),
      'photo':
          'https://images.pexels.com/photos/220453/pexels-photo-220453.jpeg?auto=compress&cs=tinysrgb&dpr=2&h=650&w=940'
    },
    {
      'id': 'asfgasga41',
      'display': 'markT',
      'full_name': 'Mark Twain',
      'photo':
          'https://images.pexels.com/photos/220453/pexels-photo-220453.jpeg?auto=compress&cs=tinysrgb&dpr=2&h=650&w=940'
    },
    {
      'id': 'asfsaf451a',
      'display': 'JhonL',
      'full_name': 'Jhon Legend',
      'photo':
          'https://images.pexels.com/photos/220453/pexels-photo-220453.jpeg?auto=compress&cs=tinysrgb&dpr=2&h=650&w=940'
    },
  ];

  @override
  void initState() {
    print(key.currentState?.controller?.markupText);
    key.currentState?.controller?.addListener(() {
      print(key.currentState?.controller?.text);
    });

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.title),
      ),
      body: Column(
        mainAxisAlignment: MainAxisAlignment.end,
        children: <Widget>[
          ValueListenableBuilder(
              valueListenable:
                  key.currentState?.showSuggestions ?? ValueNotifier(false),
              builder: (BuildContext context, bool show, Widget child) {
                return _showList
                    ? Container(
                        color: Colors.purple,
                        child: ListView.builder(
                            itemCount: _users.length,
                            shrinkWrap: true,
                            itemBuilder: (context, index) {
                              return GestureDetector(
                                onTap: () {
                                  key.currentState.addMention(_users[index]);
                                },
                                child: Container(
                                  color: Colors.amber,
                                  child: Text(_users[index]['display']),
                                ),
                              );
                            }),
                      )
                    : Container();
              }),
          RaisedButton(
            child: Text('Get Text'),
            onPressed: () {
              print(key.currentState.controller.markupText);
            },
          ),
          Container(
            padding: EdgeInsets.only(bottom: 40.0),
            width: MediaQuery.of(context).size.width,
            height: 120.0,
            child: FlutterMentions(
              key: key,
              decoration: InputDecoration(hintText: 'Enter here...'),
              suggestionPosition: SuggestionPosition.Top,
              onMarkupChanged: (val) {
                // print(val);
              },
              onSuggestionVisibleChanged: (val) {
                setState(() {
                  _showList = val;
                });
              },
              onSearchChanged: (trigger, value, show) {
                print('again | $trigger | $value | $show');
              },
              hideSuggestionList: true,
              onEditingComplete: () {
                key.currentState.controller.clear();
                // key.currentState.controller.text = '';
              },
              mentions: [
                Mention(
                    trigger: '@',
                    style: TextStyle(
                      color: Colors.amber,
                    ),
                    data: _users,
                    matchAll: false,
                    suggestionBuilder: (data) {
                      return Container(
                        color: Colors.white,
                        padding: EdgeInsets.all(10.0),
                        child: Row(
                          children: <Widget>[
                            CircleAvatar(
                              backgroundImage: NetworkImage(
                                data['photo'],
                              ),
                            ),
                            SizedBox(
                              width: 20.0,
                            ),
                            Column(
                              children: <Widget>[
                                Text(data['full_name']),
                                Text('@${data['display']}'),
                              ],
                            )
                          ],
                        ),
                      );
                    }),
              ],
            ),
          ),
        ],
      ),
    );
  }
}
