part of flutter_mentions;

/// A custom implementation of [TextEditingController] to support @ mention or other
/// trigger based mentions.
class AnnotationEditingController extends TextEditingController {
  Map<String, Annotation> _mapping;
  String _pattern;

  // Generate the Regex pattern for matching all the suggestions in one.
  AnnotationEditingController(this._mapping)
      : _pattern = "(${_mapping.keys.map((key) => key).join('|')})";

  /// Can be used to get the markup from the controller directly.
  String get markupText {
    final someVal = _mapping.isEmpty
        ? text
        : text.splitMapJoin(
            RegExp('$_pattern'),
            onMatch: (Match match) {
              final mention = _mapping[match[0]] ??
                  _mapping[_mapping.keys.firstWhere((element) {
                    final reg = RegExp(element);

                    return reg.hasMatch(match[0]);
                  })];

              // Default markup format for mentions
              if (!mention.disableMarkup) {
                return mention.markupBuilder != null
                    ? mention.markupBuilder(
                        mention.trigger, mention.id, mention.display)
                    : '${mention.trigger}[__${mention.id}__](__${mention.display}__)';
              } else {
                return match[0];
              }
            },
            onNonMatch: (String text) {
              return text;
            },
          );

    return someVal;
  }

  Map<String, Annotation> get mapping {
    return _mapping;
  }

  set mapping(Map<String, Annotation> _mapping) {
    this._mapping = _mapping;

    _pattern = "(${_mapping.keys.map((key) => key).join('|')})";
  }

  @override
  TextSpan buildTextSpan({TextStyle style, bool withComposing}) {
    var children = <InlineSpan>[];

    text.splitMapJoin(
      RegExp('$_pattern'),
      onMatch: (Match match) {
        if (_mapping.isNotEmpty) {
          final mention = _mapping[match[0]] ??
              _mapping[_mapping.keys.firstWhere((element) {
                final reg = RegExp(element);

                return reg.hasMatch(match[0]);
              })];
          print(mention.display);

          children.add(
            TextSpan(
              text: match[0],
              style: style.merge(mention.style),
            ),
          );
        }

        return '';
      },
      onNonMatch: (String text) {
        children.add(TextSpan(text: text, style: style));
        return '';
      },
    );

    return TextSpan(style: style, children: children);
  }
}
